"use strict";
//
// Copyright 2016-present by Pouya Kary <pouya@kary.us>
//
Object.defineProperty(exports, "__esModule", { value: true });
//
// ──────────────────────────────────────────────── I ──────────
//  :::::: M A I N : :  :   :    :     :        :          :
// ──────────────────────────────────────────────────────────
//
//
// ─── INCLUDES ───────────────────────────────────────────────────────────────────
//
const marked = require("marked");
//
// ─── GLOBAL STORAGE ─────────────────────────────────────────────────────────────
//
var resultParagraphs = new Array();
var currentLegend = 1;
var noLinking = false;
var ids = {};
//
// ─── CONSTANTS ──────────────────────────────────────────────────────────────────
//
/**
 * The grammar fo the legend
 */
const legendGrammar = /\^([^\^])+\^/gi;
//
// ─── COMPILE ────────────────────────────────────────────────────────────────────
//
/**
 * Compiles a legend string code into HTML
 */
function compile(code, options) {
    // init
    resultParagraphs =
        new Array();
    if (options.startingIndex != undefined)
        currentLegend = options.startingIndex;
    if (options.noLinking != undefined)
        noLinking = options.noLinking;
    // compile
    for (const paragraph of getParagraphs(code))
        resultParagraphs.push(compileParagraph(paragraph));
    // render
    let renderedCode = renderParagraphTableRowArray();
    // done
    return {
        compiledSource: renderedCode,
        currentLegendNumber: currentLegend
    };
}
exports.compile = compile;
//
// ─── LOADING RESOURCES ──────────────────────────────────────────────────────────
//
function completeHTML(code) {
    return '<link rel="stylesheet" href="legend.css">\n' + code;
}
exports.completeHTML = completeHTML;
// ────────────────────────────────────────────────────────────────────────────────
//
// ──────────────────────────────────────────────────────── II ──────────
//  :::::: C O M P I L E R : :  :   :    :     :        :          :
// ──────────────────────────────────────────────────────────────────
//
//
// ─── COMPILE PARAGRAPH ──────────────────────────────────────────────────────────
//
/**
 * Compiles the legend part of the code as well as the markdown paragraphs
 */
function compileParagraph(paragraphCode) {
    // our result
    var result = {
        paragraph: '',
        legends: new Array()
    };
    // compiling
    result.paragraph = paragraphCode.replace(legendGrammar, (match) => {
        // creating the id
        let id = createUniqueId();
        // adding the legend
        let legendContent = marked(match.substring(1, match.length - 1));
        // removing <p> ... </p>
        legendContent =
            legendContent.substring(3, legendContent.length - 5);
        // adding it...
        result.legends.push(renderBulletin(id, currentLegend, legendContent));
        //returning the id
        let insiderTextOrLink;
        if (!noLinking)
            // insiderTextOrLink = `<a href="#${ id }">&dagger;${ currentLegend++ }</a>`;
            insiderTextOrLink = '&dagger;' + currentLegend++;
        else
            insiderTextOrLink = (currentLegend++).toString();
        return `<sup class="legend-sup">${insiderTextOrLink}</sup>`;
    });
    // compiling markdown
    result.paragraph = marked(result.paragraph);
    // done
    return result;
}
//
// ─── GENERATE RANDOM IDS ────────────────────────────────────────────────────────
//
function createUniqueId() {
    while (true) {
        const id = 'legend' + Math.random()
            .toString(16)
            .slice(2);
        if (ids[id] === undefined) {
            ids[id] = true;
            return id;
        }
    }
}
//
// ─── GET PARAGRAPHS ─────────────────────────────────────────────────────────────
//
function getParagraphs(code) {
    return code.split('\n\n');
}
// ────────────────────────────────────────────────────────────────────────────────
//
// ──────────────────────────────────────────────────────── III ──────────
//  :::::: R E N D E R E R : :  :   :    :     :        :          :
// ──────────────────────────────────────────────────────────────────
//
//
// ─── RENDERING THE TEXT ─────────────────────────────────────────────────────────
//
/**
 * Renders the **`paragraphTableRow[ ]`**
 */
function renderParagraphTableRowArray() {
    const result = new Array();
    for (const row of resultParagraphs)
        result.push(renderParagraph(row));
    return result.join('\n\n');
}
//
// ─── RENDER BULLETIN ────────────────────────────────────────────────────────────
//
function renderBulletin(id, currentLegend, legendContent) {
    return `<div class="legend-bullet" id="${id}">
                    <div class="legend-bullet-number">${currentLegend}</div>
                    <div class="legend-bullet-content">${legendContent}</div>
                </div>`;
}
//
// ─── RENDER PARAGRAPH ───────────────────────────────────────────────────────────
//
function renderParagraph(row) {
    return `<div class="legend-section">
                    <div class="legend-paragraph">${row.paragraph}</div>
                    <div class="legend-sidebar">${row.legends.join('\n')}</div>
                </div>`;
}
// ────────────────────────────────────────────────────────────────────────────────
