"use strict";
//
// Copyright 2016-present by Pouya Kary <pouya@kary.us>
//
Object.defineProperty(exports, "__esModule", { value: true });
//
// ─── INCLUDES ───────────────────────────────────────────────────────────────────
//
const legend = require("./legend");
const marked = require("marked");
//
// ─── ENUMS ──────────────────────────────────────────────────────────────────────
//
var language;
(function (language) {
    language[language["html"] = 0] = "html";
    language[language["markdown"] = 1] = "markdown";
    language[language["legend"] = 2] = "legend";
})(language || (language = {}));
//
// ─── GLOBAL SETTINGS ────────────────────────────────────────────────────────────
//
var currentLegendNumber = 1;
var resetLegendAtPart = false;
var legendNoLinking = false;
//
// ─── START ──────────────────────────────────────────────────────────────────────
//
function compile(pageText, options) {
    currentLegendNumber = 1;
    if (options.legendNoLinking != undefined)
        legendNoLinking = options.legendNoLinking;
    const parsedParts = parseParts(pageText);
    return compileParts(parsedParts);
}
exports.compile = compile;
//
// ─── PARSE PARTS ────────────────────────────────────────────────────────────────
//
/**
 * Parses the text file and detects the ***language parts***.
 */
function parseParts(pageText) {
    // defs
    const result = new Array();
    let currentLanguage = language.html;
    let currentValueLines = new Array();
    // functions
    function pushCleanReset() {
        if (currentValueLines.length > 0) {
            result.push({
                kind: currentLanguage,
                value: currentValueLines.join('\n')
            });
            currentLanguage =
                language.html;
            currentValueLines =
                [];
        }
    }
    // body
    pageText.split('\n').forEach(line => {
        // if not comment
        if (!line.match(/^\s*\/\/.*$/gi)) {
            // starting markdown part
            if (line.match(/^\s*---\s*(md|mdown|markdown)\s*$/i)) {
                pushCleanReset();
                currentLanguage =
                    language.markdown;
                // starting legend part
            }
            else if (line.match(/^\s*---\s*legend\s*$/i)) {
                pushCleanReset();
                currentLanguage =
                    language.legend;
                // ending part
            }
            else if (line.match(/^\s*---\s*(e|end)\s*$/i)) {
                pushCleanReset();
                // adding new lines to the part
            }
            else {
                currentValueLines.push(line);
            }
        }
    });
    // Adding last results
    pushCleanReset();
    // done
    return result;
}
exports.parseParts = parseParts;
//
// ─── COMPILE PARTS ──────────────────────────────────────────────────────────────
//
/**
 * Compiles different page parts
 */
function compileParts(parts) {
    const lines = new Array();
    parts.forEach(part => {
        switch (part.kind) {
            case language.legend:
                const legendResult = legend.compile(part.value, {
                    startingIndex: currentLegendNumber,
                    noLinking: legendNoLinking
                });
                if (!resetLegendAtPart)
                    currentLegendNumber = legendResult.currentLegendNumber;
                lines.push(legendResult.compiledSource);
                break;
            case language.markdown:
                lines.push(marked(part.value));
                break;
            default:
                lines.push(part.value);
                break;
        }
    });
    // and done...
    return lines.join('\n');
}
// ────────────────────────────────────────────────────────────────────────────────
